@@ -18,15 +18,16 @@
 
 import time
 
-from taskflow.patterns import linear_flow as lf
+import testtools
 
 import taskflow.engines
-
 from taskflow import exceptions as exc
+from taskflow.patterns import linear_flow as lf
 from taskflow import states
 from taskflow import task
 from taskflow import test
 from taskflow.tests import utils
+from taskflow.utils import eventlet_utils as eu
 
 
 class TestTask(task.Task):
@@ -132,8 +133,7 @@ class SuspendFlowTest(utils.EngineTestBase):
             ['a', 'b',
              'c reverted(Failure: RuntimeError: Woot!)',
              'b reverted(5)'])
-        with self.assertRaisesRegexp(RuntimeError, '^Woot'):
-            engine.run()
+        self.assertRaisesRegexp(RuntimeError, '^Woot', engine.run)
         self.assertEqual(engine.storage.get_flow_state(), states.REVERTED)
         self.assertEqual(
             self.values,
@@ -155,8 +155,7 @@ class SuspendFlowTest(utils.EngineTestBase):
 
         # pretend we are resuming
         engine2 = self._make_engine(flow, engine.storage._flowdetail)
-        with self.assertRaisesRegexp(RuntimeError, '^Woot'):
-            engine2.run()
+        self.assertRaisesRegexp(RuntimeError, '^Woot', engine2.run)
         self.assertEqual(engine2.storage.get_flow_state(), states.REVERTED)
         self.assertEqual(
             self.values,
@@ -182,8 +181,7 @@ class SuspendFlowTest(utils.EngineTestBase):
             AutoSuspendingTaskOnRevert(self.values, 'b')
         )
         engine2 = self._make_engine(flow2, engine.storage._flowdetail)
-        with self.assertRaisesRegexp(RuntimeError, '^Woot'):
-            engine2.run()
+        self.assertRaisesRegexp(RuntimeError, '^Woot', engine2.run)
         self.assertEqual(engine2.storage.get_flow_state(), states.REVERTED)
         self.assertEqual(
             self.values,
@@ -203,12 +201,9 @@ class SuspendFlowTest(utils.EngineTestBase):
         engine.run()
         self.assertEqual(engine.storage.get_flow_state(), states.SUSPENDED)
         # uninject engine
-        engine.storage.save(
-            engine.storage.get_uuid_by_name(engine.storage.injector_name),
-            None,
-            states.FAILURE)
-        with self.assertRaises(exc.MissingDependencies):
-            engine.run()
+        engine.storage.save(engine.storage.injector_name,
+                            None, states.FAILURE)
+        self.assertRaises(exc.MissingDependencies, engine.run)
 
 
 class SingleThreadedEngineTest(SuspendFlowTest,
@@ -228,3 +223,17 @@ class MultiThreadedEngineTest(SuspendFlowTest,
         return taskflow.engines.load(flow, flow_detail=flow_detail,
                                      engine_conf=engine_conf,
                                      backend=self.backend)
+
+
+@testtools.skipIf(not eu.EVENTLET_AVAILABLE, 'eventlet is not available')
+class ParallelEngineWithEventletTest(SuspendFlowTest,
+                                     test.TestCase):
+
+    def _make_engine(self, flow, flow_detail=None, executor=None):
+        if executor is None:
+            executor = eu.GreenExecutor()
+        engine_conf = dict(engine='parallel',
+                           executor=executor)
+        return taskflow.engines.load(flow, flow_detail=flow_detail,
+                                     engine_conf=engine_conf,
+                                     backend=self.backend)
