@@ -17,9 +17,10 @@
 #    under the License.
 
 import contextlib
-import six
 import time
 
+import six
+
 from taskflow.persistence.backends import impl_memory
 from taskflow import task
 
@@ -36,7 +37,7 @@ def make_reverting_task(token, blowup=False):
     if blowup:
 
         def blow_up(context, *args, **kwargs):
-            raise Exception("I blew up")
+            raise RuntimeError("I blew up")
 
         return task.FunctorTask(blow_up, name='blowup_%s' % token)
     else:
